PROGRAM extruec;
(*
  ______   ___    ___
 /\  _  \ /\_ \  /\_ \
 \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___        __    ___      ____
  \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\    /'__`\ /\__`\  /'___/
   \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \__/\ \L\ \\/ __ \/\____`\ 
    \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/\_\ \  __//\____/\/\____/
     \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/\/_/\ \ \/ \/___/  \/___/
                                    /\____/               \ \_\
                                    \_/__/                 \/_/
 *
 *	This program shows how to specify colors in the various different
 *	truecolor pixel formats. The example shows the same screen (a few
 *	text lines and three coloured gradients) in all the color depth
 *	modes supported by your video card. The more color depth you have,
 *	the less banding you will see in the gradients.
 *	
 *	by uo Martnez <>
 *	from an example of Allegro Game Library by Shawn Hargreaves.
 *
 *	See readme.txt for license and copyright information.
 *)

USES
  sysutils,
{ It needs some Allegro.pas units. }
  alcolor,  { Color manipulation. }
  aldraw,   { Drawing primitives. }
  algraph,  { Graphic mode configuration. }
  alkeybrd, { Keyboard input. }
  alpalete, { Color palette manipulation. }
  alsystem, { System initialization. }
  altext;   { Text drawing. }



  (* MyPrintf:
   *   Helper function. *)
  PROCEDURE MyPrintf (x, y, c: INTEGER; Fmt: PCHAR; Value: INTEGER);
  VAR
    text_buff: PCHAR;
  BEGIN
    text_buff := StrAlloc (50);
    StrFmt (text_buff, Fmt, [Value]);
    al_textout_ex (al_screen, al_font^, text_buff, x, y, c, -1);
    StrDispose (text_buff);
  END;



  (* test:
   *   Does the color test. *)
  PROCEDURE test (colordepth: INTEGER);
  VAR
    pal: AL_PALETTE;
    x: INTEGER;
  BEGIN
  { Set the screen mode }
    al_set_color_depth (colordepth);

    IF al_set_gfx_mode (AL_GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0) <> 0 THEN
      EXIT;

  { In case this is a 256 color mode, we'd better make sure that the
    palette is set to something sensible. This function generates a
    standard palette with a nice range of different colors... }
    al_generate_332_palette (@pal);
    al_set_palette (pal);

    al_acquire_screen;

    al_clear_to_color (al_screen, al_makecol (0, 0, 0));

    MyPrintf (0, 0, al_makecol (255, 255, 255), '%d bit color...', colordepth);

  { use the _makecol() function to specify RGB values... }
    MyPrintf (32,  80, al_makecol (255,   0,   0), 'Red',     0);
    MyPrintf (32, 100, al_makecol (  0, 255,   0), 'Green',   0);
    MyPrintf (32, 120, al_makecol (  0,   0, 255), 'Blue',    0);
    MyPrintf (32, 140, al_makecol (255, 255,   0), 'Yellow',  0);
    MyPrintf (32, 160, al_makecol (  0, 255, 255), 'Cyan',    0);
    MyPrintf (32, 180, al_makecol (255,   0, 255), 'Magenta', 0);
    MyPrintf (32, 200, al_makecol (128, 128, 128), 'Grey',    0);

  { or we could draw some nice smooth color gradients... }
    FOR x := 0 TO 255 DO
    BEGIN
      al_vline (al_screen, 192 + x, 112, 176, al_makecol (x, 0, 0));
      al_vline (al_screen, 192 + x, 208, 272, al_makecol (0, x, 0));
      al_vline (al_screen, 192 + x, 304, 368, al_makecol (0, 0, x));
    END;

    al_textout_centre_ex (al_screen, al_font^, '<press a key>',
			  AL_SCREEN_W DIV 2, AL_SCREEN_H - 16,
			  al_makecol (255, 255, 255), -1);

    al_release_screen;

    al_readkey();
END;



BEGIN { The program starts here. }
  IF al_init <> 0 THEN
    EXIT;
  al_install_keyboard;

{ Yry each of the possible possible color depths... }
  test ( 8);
  test (15);
  test (16);
  test (24);
  test (32);

{ Shutdown Allegro. }
  al_exit;

{ End of the program. }
END.

